@@ -6,11 +6,14 @@
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
+#include <optional>
+
 #include "screenmapper.h"
 
 #include <QScreen>
 #include <QTimer>
 
+#include <KActivities/Consumer>
 #include <KConfig>
 #include <KConfigGroup>
 #include <Plasma/Corona>
@@ -48,9 +51,11 @@ ScreenMapper::ScreenMapper(QObject *parent)
     m_screenMappingChangedTimer->setSingleShot(true);
 }
 
-void ScreenMapper::removeScreen(int screenId, const QUrl &screenUrl)
+void ScreenMapper::removeScreen(int screenId, const QString &activity, const QUrl &screenUrl)
 {
-    if (screenId < 0 || !m_availableScreens.contains(screenId))
+    const std::pair<int, QString> pair = std::make_pair(screenId, activity);
+
+    if (screenId < 0 || !m_availableScreens.contains(pair))
         return;
 
     const auto screenPathWithScheme = screenUrl.url();
@@ -59,16 +64,22 @@ void ScreenMapper::removeScreen(int screenId, const QUrl &screenUrl)
     QVector<QUrl> urlsToRemoveFromMapping;
     while (it != m_screenItemMap.constEnd()) {
         const auto name = it.key();
-        if (it.value() == screenId && name.url().startsWith(screenPathWithScheme)) {
+        if (it.value() == screenId && name.first.url().startsWith(screenPathWithScheme) && name.second == activity) {
             bool found = false;
             for (const auto &disabledUrls : qAsConst(m_itemsOnDisabledScreensMap)) {
-                found = disabledUrls.contains(name);
+                found = disabledUrls.contains(name.first);
                 if (found)
                     break;
             }
-            if (!found)
-                m_itemsOnDisabledScreensMap[screenId].append(name);
-            urlsToRemoveFromMapping.append(name);
+            if (!found) {
+                auto urlVectorIt = m_itemsOnDisabledScreensMap.find(pair);
+                if (urlVectorIt == m_itemsOnDisabledScreensMap.end()) {
+                    m_itemsOnDisabledScreensMap[pair] = {name.first};
+                } else {
+                    urlVectorIt->append(name.first);
+                }
+            }
+            urlsToRemoveFromMapping.append(name.first);
         }
         ++it;
     }
@@ -76,39 +87,41 @@ void ScreenMapper::removeScreen(int screenId, const QUrl &screenUrl)
     saveDisabledScreensMap();
 
     for (const auto &url : urlsToRemoveFromMapping)
-        removeFromMap(url);
+        removeFromMap(url, activity);
 
-    m_availableScreens.removeAll(screenId);
+    m_availableScreens.removeAll(pair);
 
     auto pathIt = m_screensPerPath.find(screenUrl);
     if (pathIt != m_screensPerPath.end() && pathIt.value().size() > 0) {
         // remove the screen for a certain url, used when switching the URL for a screen
-        pathIt->removeAll(screenId);
+        pathIt->removeAll(pair);
     } else if (screenUrl.isEmpty()) {
         // the screen was indeed removed so all references to it needs to be cleaned up
         for (auto &pathIt : m_screensPerPath) {
-            pathIt.removeAll(screenId);
+            pathIt.removeAll(pair);
         }
     }
 
     Q_EMIT screensChanged();
 }
 
-void ScreenMapper::addScreen(int screenId, const QUrl &screenUrl)
+void ScreenMapper::addScreen(int screenId, const QString &activity, const QUrl &screenUrl)
 {
-    if (screenId < 0 || m_availableScreens.contains(screenId))
+    const std::pair<int, QString> pair = std::make_pair(screenId, activity);
+
+    if (screenId < 0 || m_availableScreens.contains(pair))
         return;
 
     const auto screenPathWithScheme = screenUrl.url();
     // restore the stored locations
-    auto it = m_itemsOnDisabledScreensMap.find(screenId);
+    auto it = m_itemsOnDisabledScreensMap.find(pair);
     if (it != m_itemsOnDisabledScreensMap.end()) {
         auto items = it.value();
         for (const auto &name : it.value()) {
             // add the items to the new screen, if they are on a disabled screen and their
             // location is below the new screen's path
             if (name.url().startsWith(screenPathWithScheme)) {
-                addMapping(name, screenId, DelayedSignal);
+                addMapping(name, screenId, activity, DelayedSignal);
                 items.removeAll(name);
             }
         }
@@ -120,24 +133,25 @@ void ScreenMapper::addScreen(int screenId, const QUrl &screenUrl)
     }
     saveDisabledScreensMap();
 
-    m_availableScreens.append(screenId);
+    m_availableScreens.append(pair);
 
     // path is empty when a new screen appears that has no folderview base path associated with
     if (!screenUrl.isEmpty()) {
         auto it = m_screensPerPath.find(screenUrl);
         if (it == m_screensPerPath.end()) {
-            m_screensPerPath[screenUrl] = {screenId};
+            m_screensPerPath[screenUrl] = {pair};
         } else {
-            it->append(screenId);
+            it->append(pair);
         }
     }
 
     Q_EMIT screensChanged();
 }
 
-void ScreenMapper::addMapping(const QUrl &url, int screen, MappingSignalBehavior behavior)
+void ScreenMapper::addMapping(const QUrl &url, int screen, const QString &activity, MappingSignalBehavior behavior)
 {
-    m_screenItemMap[url] = screen;
+    m_screenItemMap[std::make_pair(url, activity)] = screen;
+
     if (behavior == DelayedSignal) {
         m_screenMappingChangedTimer->start();
     } else {
@@ -145,17 +159,33 @@ void ScreenMapper::addMapping(const QUrl &url, int screen, MappingSignalBehavior
     }
 }
 
-void ScreenMapper::removeFromMap(const QUrl &url)
+void ScreenMapper::removeFromMap(const QUrl &url, const QString &activity)
 {
-    m_screenItemMap.remove(url);
+    m_screenItemMap.remove(std::make_pair(url, activity));
+
     m_screenMappingChangedTimer->start();
 }
 
-int ScreenMapper::firstAvailableScreen(const QUrl &screenUrl) const
+int ScreenMapper::firstAvailableScreen(const QUrl &screenUrl, const QString &activity) const
 {
     auto screens = m_screensPerPath[screenUrl];
-    const auto newFirstScreen = std::min_element(screens.constBegin(), screens.constEnd());
-    return newFirstScreen == screens.constEnd() ? -1 : *newFirstScreen;
+    std::optional<int> newFirstScreen = std::nullopt;
+
+    for (const std::pair<int, QString> &screen : std::as_const(screens)) {
+        if (screen.second != activity) {
+            continue;
+        }
+
+        if (newFirstScreen.has_value()) {
+            if (newFirstScreen > screen.first) {
+                newFirstScreen = screen.first;
+            }
+        } else {
+            newFirstScreen = screen.first;
+        }
+    }
+
+    return newFirstScreen.value_or(-1);
 }
 
 void ScreenMapper::removeItemFromDisabledScreen(const QUrl &url)
@@ -189,18 +219,18 @@ void ScreenMapper::cleanup()
 }
 #endif
 
-void ScreenMapper::setCorona(Plasma::Corona *corona)
+void ScreenMapper::setCorona(Plasma::Corona *corona, const QString &activity)
 {
     if (m_corona != corona) {
         Q_ASSERT(!m_corona);
 
         m_corona = corona;
         if (m_corona) {
-            connect(m_corona, &Plasma::Corona::screenRemoved, this, [this](int screenId) {
-                removeScreen(screenId, {});
+            connect(m_corona, &Plasma::Corona::screenRemoved, this, [this, activity](int screenId) {
+                removeScreen(screenId, activity, {});
             });
-            connect(m_corona, &Plasma::Corona::screenAdded, this, [this](int screenId) {
-                addScreen(screenId, {});
+            connect(m_corona, &Plasma::Corona::screenAdded, this, [this, activity](int screenId) {
+                addScreen(screenId, activity, {});
             });
 
             auto config = m_corona->config();
@@ -208,7 +238,9 @@ void ScreenMapper::setCorona(Plasma::Corona *corona)
             const QStringList mapping = group.readEntry(QStringLiteral("screenMapping"), QStringList{});
             setScreenMapping(mapping);
             m_sharedDesktops = group.readEntry(QStringLiteral("sharedDesktops"), false);
-            readDisabledScreensMap();
+
+            const QStringList serializedMap = group.readEntry(QStringLiteral("itemsOnDisabledScreens"), QStringList{});
+            readDisabledScreensMap(serializedMap);
         }
     }
 }
@@ -216,11 +248,12 @@ void ScreenMapper::setCorona(Plasma::Corona *corona)
 QStringList ScreenMapper::screenMapping() const
 {
     QStringList result;
-    result.reserve(m_screenItemMap.count() * 2);
+    result.reserve(m_screenItemMap.count() * 3); // Match setScreenMapping()
     auto it = m_screenItemMap.constBegin();
     while (it != m_screenItemMap.constEnd()) {
-        result.append(it.key().toString());
-        result.append(QString::number(it.value()));
+        result.append(it.key().first.toString());
+        result.append(QString::number(it.value())); // Screen ID
+        result.append(it.key().second); // Activity ID
         ++it;
     }
 
@@ -229,13 +262,17 @@ QStringList ScreenMapper::screenMapping() const
 
 void ScreenMapper::setScreenMapping(const QStringList &mapping)
 {
-    QHash<QUrl, int> newMap;
+    decltype(m_screenItemMap) newMap;
     const int count = mapping.count();
-    newMap.reserve(count / 2);
-    for (int i = 0; i < count - 1; i += 2) {
-        if (i + 1 < count) {
+    const bool useDefaultActivity = count % 3 != 0; // Missing activity ID from the old config before 5.25
+    const int sizeOfParamGroup = useDefaultActivity ? 2 : 3; // Match screenMapping()
+
+    newMap.reserve(count / sizeOfParamGroup);
+    for (int i = 0; i < count - (sizeOfParamGroup - 1); i += sizeOfParamGroup) {
+        if (i + (sizeOfParamGroup - 1) < count) {
             const QUrl url = QUrl::fromUserInput(mapping[i], {}, QUrl::AssumeLocalFile);
-            newMap[url] = mapping[i + 1].toInt();
+            const QString activity = useDefaultActivity ? KActivities::Consumer().currentActivity() : mapping[i + 2];
+            newMap[std::make_pair(url, activity)] = mapping[i + 1].toInt();
         }
     }
 
@@ -245,11 +282,12 @@ void ScreenMapper::setScreenMapping(const QStringList &mapping)
     }
 }
 
-int ScreenMapper::screenForItem(const QUrl &url) const
+int ScreenMapper::screenForItem(const QUrl &url, const QString &activity) const
 {
-    int screen = m_screenItemMap.value(url, -1);
-    if (!m_availableScreens.contains(screen))
-        screen = -1;
+    const int screen = m_screenItemMap.value(std::make_pair(url, activity), -1);
+
+    if (!m_availableScreens.contains(std::make_pair(screen, activity)))
+        return -1;
 
     return screen;
 }
@@ -259,31 +297,60 @@ QUrl ScreenMapper::stringToUrl(const QString &path)
     return QUrl::fromUserInput(path, {}, QUrl::AssumeLocalFile);
 }
 
-void ScreenMapper::readDisabledScreensMap()
+QStringList ScreenMapper::disabledScreensMap() const
 {
-    if (!m_corona)
-        return;
+    QStringList serializedMap;
+    auto it = m_itemsOnDisabledScreensMap.constBegin();
+    for (; it != m_itemsOnDisabledScreensMap.constEnd(); ++it) {
+        serializedMap.append(QString::number(it.key().first)); // Screen ID
+        serializedMap.append(it.key().second); // Activity ID
+        const auto urls = it.value();
+        serializedMap.append(QString::number(urls.size()));
+        for (const auto &url : urls) {
+            serializedMap.append(url.toString());
+        }
+    }
 
-    auto config = m_corona->config();
-    KConfigGroup group(config, QStringLiteral("ScreenMapping"));
-    const QStringList serializedMap = group.readEntry(QStringLiteral("itemsOnDisabledScreens"), QStringList{});
+    return serializedMap;
+}
+
+void ScreenMapper::readDisabledScreensMap(const QStringList &serializedMap)
+{
     m_itemsOnDisabledScreensMap.clear();
     bool readingScreenId = true;
+    bool readingActivityId = true;
     int vectorSize = -1;
     int screenId = -1;
+    QString activityId;
     int vectorCounter = 0;
     for (const auto &entry : serializedMap) {
         if (readingScreenId) {
             screenId = entry.toInt();
             readingScreenId = false;
+        } else if (readingActivityId) {
+            // Missing activity ID in the old config before 5.25
+            if (entry.toInt() > 0) {
+                vectorSize = entry.toInt();
+                activityId = KActivities::Consumer().currentActivity();
+            } else { // When a string is a uuid, toInt() will return 0
+                activityId = entry;
+            }
+            readingActivityId = false;
         } else if (vectorSize == -1) {
             vectorSize = entry.toInt();
         } else {
             const auto url = stringToUrl(entry);
-            m_itemsOnDisabledScreensMap[screenId].append(url);
+            const auto pair = std::make_pair(screenId, activityId);
+            auto urlVectorIt = m_itemsOnDisabledScreensMap.find(pair);
+            if (urlVectorIt == m_itemsOnDisabledScreensMap.end()) {
+                m_itemsOnDisabledScreensMap[pair] = {url};
+            } else {
+                urlVectorIt->append(url);
+            }
             vectorCounter++;
             if (vectorCounter == vectorSize) {
                 readingScreenId = true;
+                readingActivityId = true;
                 screenId = -1;
                 vectorCounter = 0;
                 vectorSize = -1;
@@ -299,16 +366,7 @@ void ScreenMapper::saveDisabledScreensMap() const
 
     auto config = m_corona->config();
     KConfigGroup group(config, QStringLiteral("ScreenMapping"));
-    QStringList serializedMap;
-    auto it = m_itemsOnDisabledScreensMap.constBegin();
-    for (; it != m_itemsOnDisabledScreensMap.constEnd(); ++it) {
-        serializedMap.append(QString::number(it.key()));
-        const auto urls = it.value();
-        serializedMap.append(QString::number(urls.size()));
-        for (const auto &url : urls) {
-            serializedMap.append(url.toString());
-        }
-    }
+    const auto serializedMap = disabledScreensMap();
 
     group.writeEntry(QStringLiteral("itemsOnDisabledScreens"), serializedMap);
 }
